/*
	sdmoutrk.c  10/27/86

	% sedmou_Track

	The "tracking" mouse handler.
	Jumps to whatever field the mouse is pointing to.
	When a mouse button is clicked over the field, the sed Mouse Code
	is set to MOU_CLICK.

	If the field is is a different sed, and both seds have mouse handlers,
	then the first sed is exited and control is passed to the new sed.

	C-scape 3.1
	Copyright (c) 1986, 1987, 1988 by Oakland Group, Inc.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 5/15/89 jmd	added debouncing
	 6/07/89 jmd	renamed mouse codes
	 6/15/89 jmd	added additional debouncing
	 8/12/89 jmd	removed call to sed_GetBob
	 8/16/89 jdc	fixed in bob REQUEST nextwin = (jumpwin == NULL) bug
	 8/29/89 jmd	added test for 'fake' events
	 9/10/89 jmd	added mev_ClearEvent to fake event test
*/

#include <stdio.h>
#include "cscape.h"

#include "scancode.h"		/* for MOU codes */

unsigned sedmou_Track(sed, msg, mev)
	sed_type sed;			/* the sed window */
	int msg;				/* a mouse message */
	mev_struct *mev;	/* the mouse event */
/*
	Messages a mouse handler receives when its window is current
	MEV_REQUEST			someone else wants to be it
	MEV_STARTEVENT
	MEV_EVENT
	MEV_ENDEVENT

	Messages a mouse handler receives when its window is not current
	MEV_STARTOFFER		not current, mouse entered
	MEV_OFFER			not current, event in win
	MEV_ENDOFFER		not current, mouse left

*/
{
	sed_type owner, tmp;
	win_type jumpwin, nextwin;
	int 	 fld;
	ocbox 	 mbox;

	switch(msg){
	case MEV_STARTOFFER:	   	/* we're not the current window */
		win_SetDebounced(sed, mev_IsButtonDown(mev));

		/* no break, fall through */

	case MEV_OFFER:				/* we're not the current window */
		if (mev_IsFake()) {
			/* ignore 'fake' events */
			mev_ClearEvent(mev);
			break;
		}

		/* figure out where in the selected sed we want to jump to */
		mbox.toprow = mbox.botrow = mev_GetRow(mev) + sed_GetYoffset(sed);
		mbox.leftcol = mbox.rightcol = mev_GetCol(mev) + sed_GetXoffset(sed);

		if ((fld = sed_FindField(sed, &mbox, OAK_DOWN)) != -1) {
			if (fld != sed_GetFieldNo(sed)) {
				sed_GotoField(sed, fld);
			}
		}
		else if (msg != MEV_STARTOFFER) {
			/* mouse isn't over a field, ignore it (unless we've just entered) */
			mev_ClearEvent(mev);
			break;
		}

		if (!mev_IsButtonDown(mev)) {
			win_SetDebounced(sed, FALSE);
		}
		else {
			if (!win_GetDebounced(sed)) {
				win_SetDebounced(sed, TRUE);

				/* Set our mousecode to tell us that the mouse was clicked
				   when our field function is eventually called
				*/
				if (fld >= 0) {
					sed_SetMouseCode(sed, MOU_CLICK);
				}
			}
		}

		/* 	Check if we are in a bobbed sed, if so
			get our owner, then check if our owner is a bobbed
			sed, etc.  As we move up the chain of ownership,
			we call GotoField so that when our owner is activated
			control will fall right into our lap.
			We must set the current window's jumpwin to point to 
			our owner so that the current can jump to our owner
			instead of directly jumping to us.  (See REQUEST below).
		*/
		owner = sed;
		while ((tmp = bob_GetOwner(owner, &fld)) != NULL) {
			owner = tmp;
			/* tell the owner to move to this new field */
			if (fld != sed_GetFieldNo(owner)) {
				sed_GotoField(owner, fld);
			}
		}
		if (owner != sed) {
			/* Set the current window's sedwin jump window to the final owner. */
			win_Do(disp_GetCurrentWin(), SEDWM_SETJMP, owner, NULL);
		}
		break;

	case MEV_ENDOFFER: 		  		/* we're not the current window */
	case MEV_ENDEVENT: 		  		/* we're the current window */
		win_SetDebounced(sed, mev_IsButtonDown(mev));
		break;

	case MEV_STARTEVENT:
		win_SetDebounced(sed, mev_IsButtonDown(mev));

		/* no break, fall through */

	case MEV_EVENT:					/* we're the current window */
		if (mev_IsFake()) {
			/* ignore 'fake' events */
			break;
		}

		/* An event occurred */
		mbox.toprow = mbox.botrow = mev_GetRow(mev) + sed_GetYoffset(sed);
		mbox.leftcol = mbox.rightcol = mev_GetCol(mev) + sed_GetXoffset(sed);

		/* sed_FindField finds the best field in the given box */ 
		if ((fld = sed_FindField(sed, &mbox, OAK_DOWN)) != -1) {
			if (fld != sed_GetFieldNo(sed)) {
				sed_GotoField(sed, fld);
			}
		}

		if (!mev_IsButtonDown(mev)) {
			win_SetDebounced(sed, FALSE);
		}
		else {
			if (!win_GetDebounced(sed)) {
				win_SetDebounced(sed, TRUE);

				/* Set our mousecode to tell us that the mouse was clicked
				   when our field function is eventually called.
				*/
				if (fld >= 0) {
					sed_SetMouseCode(sed, MOU_CLICK);
				}
			}
		}

		/* Return MOU_HERE as a scancode to kb_Read */
		/* We return MOU_HERE even if the button hasn't been pressed
		   because we may have changed fields and we want to 
		   leave the current field immediately */
		return(MOU_HERE);

	case MEV_REQUEST:
		/*  Another window was selected by the mouse. */

		/* 	Find if a jump win has been set
			The jump window is an element of the sed window object.
			It is set during an OFFER message (see above) if the
			new window is embedded.  It is set to point to the owner
			of the new window so that we can jump to it instead
			of the new window.
			If jump window is NULL, then the new window has no owner
			and we can jump directly to it.  The mouse event structure
			(mev) contains a pointer to the new window.
		*/
			
		jumpwin = NULL;
		win_Do(sed, SEDWM_GETJMP, NULL, &jumpwin);

		/* reset the jump win */
		win_Do(sed, SEDWM_SETJMP, NULL, NULL);

		/* Are we a bobbed window? */
		if (bob_GetParent(sed) != NULL) {
			/* If we are an embedded sed we must find our owner
			   and tell her to jump to the selected window */	

			/* find our owner */
			owner = sed;
			while ((tmp = bob_GetOwner(owner, &fld)) != NULL) {
				owner = tmp;
			}

			if (jumpwin != NULL) {
				/* tell owner to jump to jump window (new window is a bob) */
				sed_SetNextWin(owner, (nextwin = jumpwin));
			}
			else {
				/* tell owner jump to mouse window (new window is not a bob) */
				sed_SetNextWin(owner, (nextwin = mev_GetWin(mev)));
			}
		}
		else {
			/* We're not a bobbed window */
			if (jumpwin != NULL) {
				/* jump to jump window (new window is a bob) */
				sed_SetNextWin(sed, (nextwin = jumpwin));
			}
			else {
				/* jump to mouse window (new window is not a bob) */
				sed_SetNextWin(sed, (nextwin = mev_GetWin(mev)));
			}
		}

		/* Top the new window */
		win_Top(nextwin);

		/* Return MOU_THERE as a scancode to kb_Read */
		return(MOU_THERE);
	}

	return(MOU_IGNORE);		/* Ignore this mouse event */
}

